Lms

Struct Lms 

#[repr(C)]
pub struct Lms<M, T> { pub long: T, pub medium: T, pub short: T, pub meta: PhantomData<M>, }
Generic LMS.

LMS represents the response of the eye’s cone cells. L, M and S are for “long”, “medium” and “short” wavelengths, roughly corresponding to red, green and blue. Many newer mentions of an LMS representation use the letters R, G and B instead (or sometimes ρ, γ, β), but this library sticks to LMS to differentiate it from Rgb.

The LMS color space is a model of the physiological response to color stimuli. It has some mathematical shortcomings that Xyz improves on, such as severe spectral sensitivity overlap between L, M and S. Despite this, LMS has a lot of uses, include chromatic adaptation and emulating different types of color vision deficiency, and it’s sometimes part of the conversion process between other color spaces.

Creating a Value

An LMS value is often derived from another color space, through a conversion matrix. Two such matrices are VonKries and Bradford, and Palette offers type aliases in the lms module to make using them a bit more convenient. It’s of course also possible to simply use Lms::new, but it may not be as intuitive.

use palette::{
    lms::{Lms, VonKriesLms, matrix::VonKries},
    white_point::D65,
    Srgb, FromColor
};

let von_kries_lms = Lms::<VonKries, f32>::new(0.1, 0.2, 0.3);
let von_kries_d65_lms = VonKriesLms::<D65, f32>::new(0.1, 0.2, 0.3);

// `new` is also `const`:
const VON_KRIES_LMS: Lms<VonKries, f32> = Lms::new(0.1, 0.2, 0.3);

// Von Kries LMS from sRGB:
let lms_from_srgb = VonKriesLms::<D65, f32>::from_color(Srgb::new(0.3f32, 0.8, 0.1));

// It's also possible to convert from (and to) arrays and tuples:
let lms_from_array = VonKriesLms::<D65, f32>::from([0.1, 0.2, 0.3]);
let lms_from_tuple = VonKriesLms::<D65, f32>::from((0.1, 0.2, 0.3));

Fields

long: T

Stimulus from long wavelengths, or red, or ρ. The typical range is between 0.0 and 1.0, but it doesn’t have an actual upper bound.

medium: T

Stimulus from medium wavelengths, or green, or γ. The typical range is between 0.0 and 1.0, but it doesn’t have an actual upper bound.

short: T

Stimulus from short wavelengths, or blue, or β. The typical range is between 0.0 and 1.0, but it doesn’t have an actual upper bound.

meta: PhantomData<M>

Type level meta information, such as reference white, or which matrix was used when converting from XYZ.

Implementations

Source

impl<M, T> Lms<M, T>

pub const fn new(long: T, medium: T, short: T) -> Self

Create a new LMS color.

pub fn into_format<U>(self) -> Lms<M, U>
where U: FromStimulus<T>,

Convert the LMS components into another number type.

use palette::{
    lms::VonKriesLms,
    white_point::D65,
};

let lms_f64: VonKriesLms<D65, f64> = VonKriesLms::new(0.3f32, 0.7, 0.2).into_format();
pub fn from_format<U>(color: Lms<M, U>) -> Self
where T: FromStimulus<U>,

Convert the LMS components from another number type.

use palette::{
    lms::VonKriesLms,
    white_point::D65,
};

let lms_f64 = VonKriesLms::<D65, f64>::from_format(VonKriesLms::new(0.3f32, 0.7, 0.2));
pub fn into_components(self) -> (T, T, T)

Convert to a (long, medium, short) tuple.

pub fn from_components((long, medium, short): (T, T, T)) -> Self

Convert from a (long, medium, short) tuple.

pub fn with_meta<NewM>(self) -> Lms<NewM, T>

Changes the meta type without changing the color value.

This function doesn’t change the numerical values, and thus the stimuli it represents in an absolute sense. However, the appearance of the color may not be the same. The effect may be similar to taking a photo with an incorrect white balance.

Source

impl<M, T> Lms<M, T>
where T: Zero,

pub fn min_short() -> T

Return the short value minimum.

pub fn min_medium() -> T

Return the medium value minimum.

pub fn min_long() -> T

Return the long value minimum.

Source

impl<M, T> Lms<M, T>

pub fn matrix_from_xyz() -> Matrix3<Xyz<M::XyzMeta, T>, Self>

Produce a conversion matrix from Xyz to Lms.

Source

impl<M, T> Lms<M, &T>

pub fn copied(&self) -> Lms<M, T>
where T: Copy,

Get an owned, copied version of this color.

pub fn cloned(&self) -> Lms<M, T>
where T: Clone,

Get an owned, cloned version of this color.

Source

impl<M, T> Lms<M, &mut T>

pub fn set(&mut self, value: Lms<M, T>)

Update this color with new values.

pub fn as_refs(&self) -> Lms<M, &T>

Borrow this color’s components as shared references.

pub fn copied(&self) -> Lms<M, T>
where T: Copy,

Get an owned, copied version of this color.

pub fn cloned(&self) -> Lms<M, T>
where T: Clone,

Get an owned, cloned version of this color.

Source

impl<M, C> Lms<M, C>

pub fn iter<'a>(&'a self) -> <&'a Self as IntoIterator>::IntoIter
where &'a Self: IntoIterator,

Return an iterator over the colors in the wrapped collections.

pub fn iter_mut<'a>(&'a mut self) -> <&'a mut Self as IntoIterator>::IntoIter
where &'a mut Self: IntoIterator,

Return an iterator that allows modifying the colors in the wrapped collections.

pub fn get<'a, I, T>( &'a self, index: I, ) -> Option<Lms<M, &'a <I as SliceIndex<[T]>>::Output>>
where T: 'a, C: AsRef<[T]>, I: SliceIndex<[T]> + Clone,

Get a color, or slice of colors, with references to the components at index. See slice::get for details.

pub fn get_mut<'a, I, T>( &'a mut self, index: I, ) -> Option<Lms<M, &'a mut <I as SliceIndex<[T]>>::Output>>
where T: 'a, C: AsMut<[T]>, I: SliceIndex<[T]> + Clone,

Get a color, or slice of colors, that allows modifying the components at index. See slice::get_mut for details.

Source

impl<M, T> Lms<M, Vec<T>>

pub fn with_capacity(capacity: usize) -> Self

Create a struct of vectors with a minimum capacity. See Vec::with_capacity for details.

pub fn push(&mut self, value: Lms<M, T>)

Push an additional color’s components onto the component vectors. See Vec::push for details.

pub fn pop(&mut self) -> Option<Lms<M, T>>

Pop a color’s components from the component vectors. See Vec::pop for details.

pub fn clear(&mut self)

Clear the component vectors. See Vec::clear for details.

pub fn drain<R>(&mut self, range: R) -> Iter<Drain<'_, T>, M>
where R: RangeBounds<usize> + Clone,

Return an iterator that moves colors out of the specified range.